<?php
namespace Teknisa\FAI\Modules\Parametros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;

class Fai01100ParametrosTipoFaturamento extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'fai01100_filial_tipo_faturamento';

    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;
    protected $util;
    protected $newCode;
    protected $connection;
    
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $util, $newCode, $connection) {
        parent::__construct($crudService, $parameterBag);
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->util = $util;
        $this->newCode = $newCode;
        $this->connection = $connection;
    }
    
    public function loadFiliais(DTO\Request\DataSet $request, DTO\Response $response) {
        $data = $request->getDataSet()->getRows();
        $resp = array();
        foreach($data as $row) {
            $conditions = array(
                array(
                    "columnName" => "CDTIPOFATUR",
                    "operator"   => "=",
                    "value"      => $row['CD']
                )
            );
            $filterCriteria = new FilterCriteria("fai01100_filial_tipo_faturamento", $conditions);
            $newFilter = $this->crudService->populateParameterBag($filterCriteria, array("CDTIPOFATUR"));
            $result = $this->dataSourceManager->findBy($newFilter);
            $filiais = $result->getRows();
            foreach($filiais as $filial) {
                $filial['PARENT'] = $row['ID'];
                $filial['ID'] = $filial['CDFILIAL'];
                $filial['NAME'] = $filial['CDFILIAL'] . ' | ' .$filial['NMFILIAL'];
                $filial['WINDOW'] = strtolower($row['ID']).'FilialWidget';
                $resp[] = $filial;
            }
        }
        $dataSet = new Dataset('filiaisTipoFat', $resp);
        $response->addDataSet($dataSet, 'filiaisTipoFat');
    }
    
    public function saveOperacao(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "tipooperdevolfat";
        $row = $request->getDataSet()->getRows();
        $error = false;
        
        try{
            $this->connection->beginTransaction();
            // Verifica se já possui dados no banco.
            foreach ($row as $data){
                $validDuplicate = 'SELECT * FROM TIPOOPERDEVOLFAT WHERE CDTIPOOPER = :CDTIPOOPER';
                $params = array(
                    'CDTIPOOPER'  => $data['CDTIPOOPER']
                );
                $duplicate = $this->entityManager->getConnection()->fetchAll($validDuplicate, $params);
                if(!empty($duplicate)){
                    $error = true;
                    $response->addDataSet(new DataSet ('validDuplicate', array(true)));
                }
            }
            
            if(!$error){
                foreach($row as $data){
                    $data['NRTIPOOPERDEVOLFAT'] = $this->newCode->getCodeByTableName('TIPOOPERDEVOLFAT', $this->environment->getNrorgTrab(), 1, 10, true);
                }
                
                parent::save($request, $response);
                $this->connection->commit();
                $response->addDataSet(new DataSet ('saveOperacao', array(true)));
            }
        }catch(\Exception $e){                
            $this->connection->rollback();
            throw $e;
        }
    }
}